// vim: foldmethod=marker
dojo.require("dojo.lang");


flock.xul.ClickSplitter = function (aNode, aOpts) {/*{{{*/
    this.context = this;
    this._clickTimers = [];
    flock.xul.Node.call(this, aNode, aOpts);
};/*}}}*/
dojo.inherits(flock.xul.ClickSplitter, flock.xul.Node);
dojo.lang.extend(flock.xul.ClickSplitter, {
    catchExtraClicks: false,
    _clickTimers: null,
    context: null,
    // threshold to wait before sending single clicks
    dblClickThreshold: 200,
    suppressNextClick: false,
    modifiers: {/*{{{*/
        "Dbl": function (aEvt) {
            return aEvt.type == "dblclick";
        },
        "Shift": function (aEvt) {
            return aEvt.shiftKey;
        },
        "Ctrl": function (aEvt) {
            return aEvt.ctrlKey;
        },
        "Alt": function (aEvt) {
            return aEvt.altKey;
        },
        "Meta": function (aEvt) {
            return aEvt.metaKey;
        },
        "Mdl": function (aEvt) {
            return aEvt.button == 1;
        }
    },/*}}}*/
    split: function (aEvent) {/*{{{*/
        var mods = [];
        for (mod in this.modifiers) {
            if (this.modifiers[mod](aEvent)) {
                mods.push(mod);
            }
        }
        return mods;
    },/*}}}*/
    handle: function (aEvent) {/*{{{*/
        if (this.suppressNextClick) {
            this.suppressNextClick = false;
            return;
        }
        var mods = this.split(aEvent);
        var method = "on" + mods.join('') + "Click";
        var context = this.context;
        var extraArgs = argsToArray(arguments, 2);
        if (context[method]) {
            extraArgs.unshift(aEvent);
            context[method].apply(context, extraArgs);
        }
    },/*}}}*/
    clearClickTimers: function () {/*{{{*/
        flock.lang.clearTimeouts(this._clickTimers);
        this._clickTimers = [];
    },/*}}}*/
    attach: function () {/*{{{*/
        if (this.isAttached) return;
        dojo.event.connect(this.mNode, "onclick", this, "handle");   
        dojo.event.connect(this.mNode, "ondblclick", this, "handle");   
        this._isAttached = true;
    },/*}}}*/
    detach: function () {/*{{{*/
        if (!this.isAttached) return;
        dojo.event.disconnect(this.mNode, "onclick", this, "handle");   
        dojo.event.disconnect(this.mNode, "ondblclick", this, "handle");   
        this._isAttached = false;
    }/*}}}*/
});
